﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WobbrockLib.Controls
{
	/// <summary>
	/// The custom numeric up down controls provide an event for when their actual buttons
	/// were clicked. That event takes these arguments to indicate which up/down button was ,
	/// clicked and what the value of the control was before and after it was clicked.
	/// </summary>
	public class UpDownClickEventArgs : UpDownEventArgs
	{
		/// <summary>
		/// Constructor.
		/// </summary>
		public UpDownClickEventArgs(UpDownButton buttonID, decimal valueBefore, decimal valueAfter)
			: base((int) buttonID)
		{
			this.ValueBefore = valueBefore;
			this.ValueAfter = valueAfter;
		}

		/// <summary>
		/// Gets the button ID indicating whether the up or down button was clicked on the control.
		/// </summary>
		public new UpDownButton ButtonID
		{
			get { return (UpDownButton) base.ButtonID; }
		}

		/// <summary>
		/// Gets the value before the up or down button was clicked.
		/// </summary>
		public decimal ValueBefore { get; }

		/// <summary>
		/// Gets the value after the up or down button was clicked.
		/// </summary>
		public decimal ValueAfter { get; }

	}
}
