﻿using System;

namespace WobbrockLib.Optimization
{
    /// <summary>
    /// A delegate, which in C++ parlance is a function pointer type, which defines a
    /// callback function used by optimization classes to indicate progress.
    /// </summary>
    /// <param name="value">
    /// A datum indicating the progress of the particular optimization search algorithm 
    /// being used. The meaning of this datum will be specific to the type of optimization
    /// algorithm employed.
    /// </param>
    /// <remarks>
    /// The value reported is specific to the type of optimization algorithm being used. For 
    /// a hill climbing algorithm, it might be the number of upward moves made. For a simulated 
    /// annealing algorithm, it might be the decreasing temperature. For a genetic algorithm, 
    /// it might be the number of generations that have transpired.
    /// </remarks>
    public delegate void OptimizationCallback(double value);
}