﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.VisualBasic;
using WobbrockLib.Extensions;
using WobbrockLib.Types;
using WobbrockLib.Controls;

namespace Test
{
    public partial class ClusterTestForm : Form
    {
		private List<List<PointR>> _clusters; // for drawing connections
        private string _salpha; // G-means

        /// <summary>
        /// Form for testing the ClusterEx extension utility functions.
        /// </summary>
        public ClusterTestForm()
        {
            InitializeComponent();
            _salpha = ".01";
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void KMeansForm_Load(object sender, EventArgs e)
        {
            Plot.Series axes = new Plot.Series("Axes", Color.Black, Color.Black, false, false);
            axes.AddPoint(0f, 0f);
			axes.AddPoint(100f, 100f);
			//axes.AddPoint(2f, 2f);
			pltGraph.AddSeries(axes);
            pltGraph.Invalidate();

			//Plot.Series zero = new Plot.Series("X-Axis", Color.LightGray, Color.LightGray, false, true);
			//zero.AddPoint(0f, 0f);
			//zero.AddPoint(2f, 0f);
			//pltGraph.AddSeries(zero);

			// Debugging test for Anderson-Darling statistic. See http://www.theriac.org/DeskReference/viewDocument.php?id=60.
			//double[] x = { 338.7, 308.5, 317.7, 313.1, 322.7, 294.2 };
			//double A2, CV;
			//bool nonNormal = StatsEx.AndersonDarling(x, .05, out A2, out CV);

			//pltGraph.XAxisDecimals = 4;
			//pltGraph.YAxisDecimals = 4;
			//pltGraph.XAxisTicks = 20;
			//pltGraph.YAxisTicks = 20;
			//Plot.Series sd = new Plot.Series("Points", Color.Red, Color.Red, false, true);
			//for (double x = 0.0; x <= 2.0; x += 0.05)
			//{
			//	double y = StatsEx.NormalDistribution(x, 0, 0.25) * 0.6265;
			//	//double y = StatsEx.ExponentialDistribution(x, 1.5) - 0.5;
			//	sd.AddPoint(x, y);
			//}
			//pltGraph.AddSeries(sd);
			//pltGraph.Invalidate();
		}

        /// <summary>
        /// Plot a handful of points in various clusters on the graph.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdPlot_Click(object sender, EventArgs e)
        {
            string s1 = Interaction.InputBox("How many random clusters?", "Clusters", "4");
            string s2 = Interaction.InputBox("Max random points per cluster?", "Max Points", "20");
            if (Int32.TryParse(s1, out int c) && Int32.TryParse(s2, out int p))
            {
                // remove all series but restore the "Axes" series
                Plot.Series axes = pltGraph[0];
                pltGraph.ClearSeries();
                pltGraph.AddSeries(axes);

				_clusters?.Clear(); // clear the results of any clustering algorithm

                // create 'c' clusters with random numbers of points [1,p] spread (-10,+10) from their random centers
                for (int i = 0; i < c; i++)
                {
                    PointF center = new PointF(RandomEx.Integer(0, 100), RandomEx.Integer(0, 100));
                    int pts = RandomEx.Integer(1, p);
                    Plot.Series series = new Plot.Series(center.ToString(), RandomEx.Color(155, 215), Color.Black, true, false);

                    for (int j = 0; j < pts; j++)
                        series.AddPoint(center.X + RandomEx.Integer(-10, +10), center.Y + RandomEx.Integer(-10, +10));
                    pltGraph.AddSeries(series);
                }
            }
            pltGraph.Invalidate();

            // update the label
            Plot.Series centers = pltGraph.GetSeries("Cluster Centers");
            lblPoints.Text = String.Format("No. Points: {0}", pltGraph.NumPoints - 2 - (centers != null ? centers.NumPoints : 0)); // subtract 2 points for the axes
            lblClusters.Text = "No. Clusters Found: "; // clear when having just plotted new points
        }

		/// <summary>
		/// Draw connections between clustered points.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pltGraph_Paint(object sender, PaintEventArgs e)
		{
			if (_clusters?.Count > 0)
			{
				for (int i = 0; i < _clusters.Count; i++)
				{
					Pen pen = new Pen(RandomEx.Color(145, 215));
					for (int j = 0; j < _clusters[i].Count; j++)
					{
						for (int k = j + 1; k < _clusters[i].Count; k++)
						{
							PointF p1 = pltGraph.Logical2Device(_clusters[i][j]);
							PointF p2 = pltGraph.Logical2Device(_clusters[i][k]);
							e.Graphics.DrawLine(pen, p1, p2);
						}
					}
					pen.Dispose();
				}
			}
		}

		/// <summary>
		/// Run k-means++ clustering on the points on the graph.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void cmdKmeans_Click(object sender, EventArgs e)
        {
            Plot.Series cc = pltGraph.GetSeries("Cluster Centers");
            string s1 = Interaction.InputBox("What value for k?", "Find Clusters", cc != null ? cc.NumPoints.ToString() : "4");
            if (Int32.TryParse(s1, out int k) && k > 0)
            {
                // remove any prior cluster centers
                pltGraph.RemoveSeries(cc?.Name);

                // build the list of points to cluster from the graph itself
                List<PointR> points = new List<PointR>();
                for (int i = 1; i < pltGraph.NumSeries; i++) // start at i=1 to skip the "Axes" series
                    for (int j = 0; j < pltGraph[i].NumPoints; j++)
                        points.Add(pltGraph[i][j]);

                // run k-means++
                List<PointR> C = ClusterEx.KMeansPlusPlus(points, k); // get k-means++ initial centers
                _clusters = ClusterEx.KMeans(points, C); // k-means!

                // add the cluster centers as red dots
                Plot.Series centers = new Plot.Series("Cluster Centers", Color.Red, Color.Black, true, false);
                centers.AddPoints(C); // plot centers
                pltGraph.InsertSeries(1, centers); // insert second so drawn after axes but on top of points
                pltGraph.Invalidate();

                // update the label
                lblClusters.Text = String.Format("No. Clusters Found: {0}", _clusters?.Count);
            }
        }

        /// <summary>
        /// Run G-means clustering on the points on the graph.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdGmeans_Click(object sender, EventArgs e)
        {
            string s = Interaction.InputBox("What value for alpha (.10, .05, .025, .01, .005)?", "Alpha", _salpha);
            if (Double.TryParse(s, out double alpha) && alpha > 0.0)
            {
                _salpha = s; // save

                // remove any prior cluster centers
                pltGraph.RemoveSeries("Cluster Centers");

                // build the list of points to cluster from the graph itself
                List<PointR> points = new List<PointR>();
                for (int i = 1; i < pltGraph.NumSeries; i++) // start at i=1 to skip the "Axes" series
                    for (int j = 0; j < pltGraph[i].NumPoints; j++)
                        points.Add(pltGraph[i][j]);

				//
				// The next lines implement G-means. The user was prompted for an alpha value. One could also
				// determine alpha based roughly on the following table:
				// No. Points (use alpha value)
				//   1 to 400 (α = .10)
				//   401 to 1000 (α = .05)
				//   1001 to 2000 (α = .025)
				//   2001 to 5000 (α = .01)
				//   5001+ (α = .005)
				//
				_clusters = ClusterEx.GMeans(points, alpha, out List<PointR> C); // G-means!

                // add the cluster centers as red dots
                Plot.Series centers = new Plot.Series("Cluster Centers", Color.Red, Color.Black, true, false);
                centers.AddPoints(C); // plot centers
                pltGraph.InsertSeries(1, centers); // insert second so drawn after axes but on top of points
                pltGraph.Invalidate();

                // update the label
                lblClusters.Text = String.Format("No. Clusters Found: {0}", _clusters?.Count);
            }
        }

        /// <summary>
        /// Run correlation clustering as the CC-Pivot algorithm on the points on the graph.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdCCPivot_Click(object sender, EventArgs e)
        {
            Plot.Series cc = pltGraph.GetSeries("Cluster Centers");
            string s = Interaction.InputBox(
                "How many correlation clustering iterations do you want?", 
                "Iterations", 
                ((pltGraph.NumPoints - 2 - (cc != null ? cc.NumPoints : 0)) * 100).ToString()
                );

            if (Int32.TryParse(s, out int n) && n > 0)
            {
                // remove any prior cluster centers
                pltGraph.RemoveSeries(cc?.Name);

                // build the list of points to cluster from the graph itself
                List<PointR> points = new List<PointR>();
                for (int i = 1; i < pltGraph.NumSeries; i++) // start at i=1 to skip the "Axes" series
                    for (int j = 0; j < pltGraph[i].NumPoints; j++)
                        points.Add(pltGraph[i][j]);

                // Iterative correlation clustering as CC-Pivot
                _clusters = ClusterEx.IterativeCCPivot(points, n, out List<PointR> C); // CC-Pivot!

                // add the cluster centers as red dots
                Plot.Series centers = new Plot.Series("Cluster Centers", Color.Red, Color.Black, true, false);
                centers.AddPoints(C); // plot centers
                pltGraph.InsertSeries(1, centers); // insert second so drawn after axes but on top of points
                pltGraph.Invalidate();

                // update the label
                lblClusters.Text = String.Format("No. Clusters Found: {0}", _clusters?.Count);
            }
        }

    }
}
