﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;
using Microsoft.VisualBasic;
using WobbrockLib;
using WobbrockLib.Types;
using WobbrockLib.Extensions;
using WobbrockLib.Controls;
using WobbrockLib.Data;

namespace Test
{
    public partial class GraphTestForm : Form
    {
        /// <summary>
        /// 
        /// </summary>
        public struct IDRectangleF
        {
            public static IDRectangleF Empty;
            public int ID;
            public RectangleF Rect;
        }

        private const float CIRCLE_SIZE = 20.0f;
        private Graph _dgraph;
        private List<IDRectangleF> _circles;

        /// <summary>
        /// 
        /// </summary>
        public GraphTestForm()
        {
            InitializeComponent();
            this.DoubleBuffered = true;

            _dgraph = new Graph();
            _dgraph.AddNode();

            _circles = new List<IDRectangleF>();
            LayoutGraph();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void GraphTestForm_Resize(object sender, EventArgs e)
        {
            LayoutGraph();
            Invalidate();
        }

        /// <summary>
        /// 
        /// </summary>
        private void LayoutGraph()
        {
            _circles.Clear();

            if (_dgraph == null || _dgraph.Count == 0)
                return;

            double radius = Math.Min(ClientRectangle.Width / 2.0, ClientRectangle.Height / 2.0) - 2.0 * SystemInformation.MenuHeight; // radius of whole graph
            PointR center = new PointR(ClientRectangle.Width / 2.0, ClientRectangle.Height / 2.0); // center point of drawing canvas

            double radians = 0.0; // start at 0 radians
            double step = (2.0 * Math.PI) / _dgraph.Count; // each radian step

            for (int i = 0; i < _dgraph.Count; i++)
            {
                PointR p = GeotrigEx.PointFromVector(center, radius, radians);
                RectangleF rect = new RectangleF((float) p.X - CIRCLE_SIZE / 2f, (float) p.Y - CIRCLE_SIZE / 2f, CIRCLE_SIZE, CIRCLE_SIZE);
                IDRectangleF idc = IDRectangleF.Empty;
                idc.Rect = rect; // store the node's rectange
                idc.ID = _dgraph[i].ID; // store the node's ID
                _circles.Add(idc);
                radians += step;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void GraphTestForm_Paint(object sender, PaintEventArgs e)
        {
            for (int i = 0; i < _circles.Count; i++) // go through every circle
            {
                for (int j = 0; j < _dgraph.Count; j++) // need to find, for each circle, its node in the graph data struct
                {
                    if (_circles[i].ID == _dgraph[j].ID) // found a graph node with the given ID
                    {
                        List<Graph.Connection> cnxns = _dgraph[j].GetOutgoingConnections(true); // get all the connections out of this node
                        for (int k = 0; k < cnxns.Count; k++) // go over each connection coming from this node
                        {
                            for (int m = 0; m < _circles.Count; m++) // find the circle with the ID for each connection
                            {
                                if (cnxns[k].To.ID == _circles[m].ID) // found the circle
                                {
                                    if (cnxns[k].To == _dgraph[j]) // node connects to itself
                                    {
                                       e.Graphics.DrawEllipse(Pens.Green, _circles[i].Rect.X + _circles[i].Rect.Width / 2f, _circles[i].Rect.Y + _circles[i].Rect.Height / 2f, CIRCLE_SIZE * 1.5f, CIRCLE_SIZE * 1.5f);
                                    }
                                    else // different nodes
                                    {
                                        PointF p1 = new PointF(_circles[i].Rect.X + _circles[i].Rect.Width / 2f, _circles[i].Rect.Y + _circles[i].Rect.Height / 2f); // start
                                        PointF p2 = new PointF(_circles[m].Rect.X + _circles[m].Rect.Width / 2f, _circles[m].Rect.Y + _circles[m].Rect.Height / 2f); // end
                                        double radians = GeotrigEx.Angle(p1, p2, true);
                                        double length = GeotrigEx.Distance(p1, p2);
                                        PointF p3 = (PointF) GeotrigEx.PointFromVector(p1, length / 2.0, radians + Math.PI / 12.0);
                                        e.Graphics.DrawCurve(Pens.Green, new PointF[] { p1, p3, p2 }, 0.5f);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            // paint the circular nodes themselves
            Font font = new Font(FontFamily.GenericSansSerif, CIRCLE_SIZE / 2f, FontStyle.Regular);
            for (int i = 0; i < _circles.Count; i++)
            {
                e.Graphics.FillEllipse(Brushes.LightBlue, _circles[i].Rect); // node fill
                e.Graphics.DrawEllipse(Pens.Blue, _circles[i].Rect); // node outline
                e.Graphics.DrawString(_circles[i].ID.ToString(), font, Brushes.Black, (float) _circles[i].Rect.X, (float) _circles[i].Rect.Y); // node ID label
            }
            font.Dispose();
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniAdd_Click(object sender, EventArgs e)
        {
            string s = Interaction.InputBox("Add node with ID?", "Add Node", _dgraph.Count.ToString(), this.Left + this.Width / 3, this.Top + this.Height / 3);
            int id;
            if (Int32.TryParse(s, out id))
            {
                if (_dgraph.AddNode(new Graph.Node(id)))
                {
                    LayoutGraph();
                    Invalidate();
                }
                else PositionableMessageBox.Show(this, new Point(this.Left + this.Width / 3, this.Top + this.Height / 3), false, "The ID you supply for this node must be unique.", "Non-Unique ID", MessageBoxButtons.OK, MessageBoxIcon.Error, true);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniAddMany_Click(object sender, EventArgs e)
        {
            string s = Interaction.InputBox("Add how many nodes?", "Add Many Nodes", "10", this.Left + this.Width / 3, this.Top + this.Height / 3);
            int n;
            if (Int32.TryParse(s, out n))
            {
                for (int i = 0; i < n; i++)
                    _dgraph.AddNode();
                LayoutGraph();
                Invalidate();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniRemove_Click(object sender, EventArgs e)
        {
            string s = Interaction.InputBox("Remove which node ID?", "Remove Node", (_dgraph.Count - 1).ToString(), this.Left + this.Width / 3, this.Top + this.Height / 3);
            int id;
            if (Int32.TryParse(s, out id))
            {
                if (_dgraph.RemoveNode(id))
                {
                    LayoutGraph();
                    Invalidate();
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniRemoveAll_Click(object sender, EventArgs e)
        {
            _dgraph.RemoveAllNodes();
            LayoutGraph();
            Invalidate();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniConnect_Click(object sender, EventArgs e)
        {
            if (_dgraph.Count > 0)
            {
                string sfrom = Interaction.InputBox("Connect from which node ID?", "From Node", (_dgraph.Count - 1).ToString(), this.Left + this.Width / 3, this.Top + this.Height / 3);
                if (sfrom != String.Empty)
                {
                    string sto = Interaction.InputBox("Connect to which node ID?", "To Node", (_dgraph.Count - 2).ToString(), this.Left + this.Width / 3, this.Top + this.Height / 3);
                    if (sto != String.Empty)
                    {
                        DialogResult yn = PositionableMessageBox.Show(this, new Point(this.Left + this.Width / 3, this.Top + this.Height / 3), false, "Connect both directions?", "Bidirectional", MessageBoxButtons.YesNo, MessageBoxIcon.Question, false);
                        int from, to;
                        if (Int32.TryParse(sfrom, out from) && Int32.TryParse(sto, out to))
                        {
                            if (_dgraph.Connect(from, to, 0.0, yn == DialogResult.Yes))
                                Invalidate();
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniDisconnect_Click(object sender, EventArgs e)
        {
            if (_dgraph.Count > 0)
            {
                string sfrom = Interaction.InputBox("Disconnect from which node ID?", "From Node", String.Empty, this.Left + this.Width / 3, this.Top + this.Height / 3);
                if (sfrom != String.Empty)
                {
                    string sto = Interaction.InputBox("Disconnect to which node ID?", "To Node", String.Empty, this.Left + this.Width / 3, this.Top + this.Height / 3);
                    if (sto != String.Empty)
                    {
                        DialogResult yn = PositionableMessageBox.Show(this, new Point(this.Left + this.Width / 3, this.Top + this.Height / 3), false, "Disconnect both directions?", "Bidirectional", MessageBoxButtons.YesNo, MessageBoxIcon.Question, false);
                        int from, to;
                        if (Int32.TryParse(sfrom, out from) && Int32.TryParse(sto, out to))
                        {
                            if (_dgraph.Disconnect(from, to, yn == DialogResult.Yes))
                                Invalidate();
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniFullyConnect_Click(object sender, EventArgs e)
        {
            DialogResult yn = PositionableMessageBox.Show(this, new Point(this.Left + this.Width / 3, this.Top + this.Height / 3), false, "Include circular connections?", "Circular Connections", MessageBoxButtons.YesNo, MessageBoxIcon.Question, false);
            _dgraph.ConnectAll(yn == DialogResult.OK);
            Invalidate();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniFullyDisconnect_Click(object sender, EventArgs e)
        {
            _dgraph.DisconnectAll();
            Invalidate();
        }

        
    }
}
