﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WobbrockLib.Devices;
using WobbrockLib.Types;

namespace Test
{
    public partial class WebCamForm : Form
    {
        public WebCamForm()
        {
            InitializeComponent();
            webcamPicture.WebCamImageChanging += new EventHandler<Image>(webcamPicture_CapturedImageChanging);
            webcamPicture.WebCamImageChanged += new EventHandler<Image>(webcamPicture_CapturedImageChanged);
            cboSizeMode.SelectedIndex = 0;
        }

        private void webcamPicture_CapturedImageChanging(object sender, Image e)
        {
            // webcamPicture.Image is the old image.
            // e.Data is the new soon-to-be-set image.
            Graphics g = Graphics.FromImage(e);
            Font f = new System.Drawing.Font(FontFamily.GenericMonospace, 12f, FontStyle.Bold);
            g.DrawString("This is the client drawing this.", f, Brushes.Blue, 100, 30);
            f.Dispose();
            g.Dispose();
        }

        private void webcamPicture_CapturedImageChanged(object sender, Image e)
        {
            // e is the old image that was just replaced.
            // webcamPicture.Image is the new image just set.
        }

        private void WebCamForm_Load(object sender, EventArgs e)
        {
            webcamPicture.Connect();
            tmrUpdate.Start();
        }

        private void WebCamForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            tmrUpdate.Stop();
            webcamPicture.Disconnect();
        }

        private void cmdSnap_Click(object sender, EventArgs e)
        {
            picCapture.Image = webcamPicture.Snapshot();
        }

        private void optPreview_CheckedChanged(object sender, EventArgs e)
        {
            if (optPreview.Checked)
            {
                webcamPicture.ImageConnection = WebcamPictureBox.WebCamImageConnectionType.PreviewViaGDIPlusNoSetImage;
            }
        }

        private void optClipboard_CheckedChanged(object sender, EventArgs e)
        {
            if (optClipboard.Checked)
            {
                webcamPicture.ImageConnection = WebcamPictureBox.WebCamImageConnectionType.CaptureViaClipboardSetImage;
            }
        }

        private void cboSizeMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cboSizeMode.SelectedIndex)
            {
                case 0:
                    webcamPicture.SizeMode = PictureBoxSizeMode.Normal;
                    break;
                case 1:
                    webcamPicture.SizeMode = PictureBoxSizeMode.StretchImage;
                    break;
                case 2:
                    webcamPicture.SizeMode = PictureBoxSizeMode.AutoSize;
                    break;
                case 3:
                    webcamPicture.SizeMode = PictureBoxSizeMode.CenterImage;
                    break;
                case 4:
                    webcamPicture.SizeMode = PictureBoxSizeMode.Zoom;
                    break;
            }
        }

        private void cmdRecord_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "Audio Video Interleave (*.avi)|*.avi";
            if (sfd.ShowDialog(this) == DialogResult.OK)
            {
                // debugging
                //bool success = false;
                //if (optSmallest.Checked)
                //{
                //    success = webcamPicture.SetVideoFormat(320, 240, 16, 844715353u) // YUY2 (?)
                //                && webcamPicture.SetAudioFormat(8000, 8, 1); // 8.0 kHz
                //}
                //else if (optSmall.Checked)
                //{
                //    success = webcamPicture.SetVideoFormat(320, 240, 24, 0u) // bimap RGB 24
                //                && webcamPicture.SetAudioFormat(11025, 8, 1); // 11.025 kHz
                //}
                //else if (optDefault.Checked)
                //{
                //    success = webcamPicture.SetVideoFormat(640, 480, 24, 0u) // bitmap RGB 24
                //                && webcamPicture.SetAudioFormat(11025, 8, 1); // 11.025 kHz
                //}
                //else if (optLarge.Checked)
                //{
                //    success = webcamPicture.SetVideoFormat(640, 480, 24, 0u) // bitmap RGB 24
                //                && webcamPicture.SetAudioFormat(44100, 16, 1); // 44.10 kHz
                //}
                //if (!success)
                //{
                //    MessageBox.Show(this, "Settings did not take.", "Fail", MessageBoxButtons.OK);
                //}
                
                webcamPicture.Record(sfd.FileName, 10u, true, true, false); // 10 seconds, asynchronous
            }
        }

        private void cmdStop_Click(object sender, EventArgs e)
        {
            webcamPicture.Stop();
        }
        
        private void tmrUpdate_Tick(object sender, EventArgs e)
        {
            bool connected = webcamPicture.IsConnected;
            bool recording = webcamPicture.IsRecording;

            lblConnected.Text = connected ? "Connected" : "Disconnected";
            lblConnected.ForeColor = connected ? Color.Green : Color.Red;

            lblRecording.Text = recording ? "Recording" : "Stopped";
            lblRecording.ForeColor = recording ? Color.Red : Color.Black;
        }

        private void cmdDlgVideoFormat_Click(object sender, EventArgs e)
        {
            webcamPicture.ShowDialog(WebcamPictureBox.WebCamDialogType.VideoFormat);
        }

        private void cmdDlgVideoSource_Click(object sender, EventArgs e)
        {
            webcamPicture.ShowDialog(WebcamPictureBox.WebCamDialogType.VideoSource);
        }

        private void cmdDlgVideoDisplay_Click(object sender, EventArgs e)
        {
            webcamPicture.ShowDialog(WebcamPictureBox.WebCamDialogType.VideoDisplay);
        }

        private void cmdDlgVideoCompression_Click(object sender, EventArgs e)
        {
            webcamPicture.ShowDialog(WebcamPictureBox.WebCamDialogType.VideoCompression);
        }

    }
}
