using System;

namespace WobbrockLib.Types
{
    /// <summary>
    /// 
    /// </summary>
	public struct AverageR
	{
		private double[] _items;
		private int _next;
		private double _sum;
        private int _numAdded;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="size"></param>
        public AverageR(int size)
            : this(size, 0.0)
        {
            // do nothing
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="size"></param>
        /// <param name="seed"></param>
		public AverageR(int size, double seed)
		{
			_sum = size * seed;
			_items = new double[size];
            for (int i = 0; i < _items.Length; i++)
            {
                _items[i] = seed;
            }
			_next = 0;
            _numAdded = 0;
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="d"></param>
		public void Add(double d)
		{
			_sum -= _items[_next];
			_sum += d;
			_items[_next++] = d;
			_next %= _items.Length;
            _numAdded++;
		}

        /// <summary>
        /// 
        /// </summary>
        public int NumAdded
        {
            get { return _numAdded; }
        }

        /// <summary>
        /// 
        /// </summary>
        public double MostRecent
        {
            get { return (_next != 0) ? _items[_next - 1] : _items[_items.Length - 1]; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
		public double GetAt(int index)
		{
			return _items[index];
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="index"></param>
        /// <param name="d"></param>
		public void SetAt(int index, double d)
		{
			_sum -= _items[index];
			_sum += d;
			_items[index] = d;
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public double this[int index]
        {
            get { return GetAt(index); }
            set { SetAt(index, value); }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="d"></param>
		public void SetAll(double d)
		{
			_sum = _items.Length * d;
            for (int i = 0; i < _items.Length; i++)
            {
                _items[i] = d;
            }
			_next = 0;
            _numAdded = _items.Length;
		}

        /// <summary>
        /// 
        /// </summary>
		public void Clear()
		{
			SetAll(0.0);
            _numAdded = 0;
		}

        /// <summary>
        /// 
        /// </summary>
		public int Size
		{
			get { return _items.Length; }
		}

        /// <summary>
        /// 
        /// </summary>
		public double Average
		{
			get { return _sum / _items.Length; }
		}
	}
}
